/*
 An example digital clock using a TFT LCD screen to show the time.
 Demonstrates use of the font printing routines. (Time updates but date does not.)
 
 For a more accurate clock, it would be better to use the RTClib library.
 But this is just a demo. 
 
 This examples uses the hardware SPI only. Non-hardware SPI
 is just too slow (~8 times slower!)
 
 Based on clock sketch by Gilchrist 6/2/2014 1.0
 Updated by Bodmer
A few colour codes:
 
code	color
0x0000	Black
0xFFFF	White
0xBDF7	Light Gray
0x7BEF	Dark Gray
0xF800	Red
0xFFE0	Yellow
0xFBE0	Orange
0x79E0	Brown
0x7E0	Green
0x7FF	Cyan
0x1F	Blue
0xF81F	Pink

 */

#include <TFT_eSPI.h> // Graphics and font library for ST7735 driver chip
#include <SPI.h>

#define BTN1    35
#define BTN2     0
//#define PHDIODE 27
#define ADC_EN  14  //ADC_EN is the ADC detection enable port
#define ADC_PIN 34
#define ADC_BIAS 26
#define ADC_PHOTO 27

int vref = 1100;

TFT_eSPI tft = TFT_eSPI();  // Invoke library, pins defined in User_Setup.h

uint32_t targetTime = 0;       // for next 1 second timeout

byte oss = 99;
bool initial = 1;
byte xcolon = 0;
unsigned int colour = 0;

uint8_t hh=0;
uint8_t mm=0;
uint8_t ss=0;
bool RUN_FLAG=false;
int off_cntr=5;

void ARDUINO_ISR_ATTR btn1_isr(void *arg) {
  hh=0;
  mm=0;
  ss=0;
  RUN_FLAG=false;
  off_cntr=5;
}

void ARDUINO_ISR_ATTR btn2_isr(void *arg) {
  RUN_FLAG=!RUN_FLAG;
}

void setup(void) {
  pinMode(BTN1, INPUT);
  attachInterruptArg(BTN1, btn1_isr, NULL, FALLING);

  pinMode(BTN2, INPUT);
  attachInterruptArg(BTN2, btn2_isr, NULL, FALLING);

  //pinMode(PHDIODE, INPUT_PULLUP);
  pinMode(ADC_BIAS, INPUT_PULLUP);

  pinMode(ADC_EN, OUTPUT);
  digitalWrite(ADC_EN, HIGH);

  tft.init();
  tft.setRotation(1);
  tft.fillScreen(TFT_BLACK);

  tft.setTextColor(TFT_YELLOW, TFT_BLACK); // Note: the new fonts do not draw the background colour

  targetTime = millis() + 1000; 
}

void loop() {
  if (targetTime < millis()) {
    targetTime = millis()+1000;
    oss=ss;
    if(RUN_FLAG)
      ss++;              // Advance second
    else
      oss=ss-1;
    
    if (ss==60) {
      ss=0;
      //omm = mm;
      mm++;            // Advance minute
      if(mm>59) {
        mm=0;
        hh++;          // Advance hour
        if (hh>23) {
          hh=0;
        }
      }
    }

    if (ss==0 || initial) {
      initial = 0;
      tft.setTextColor(TFT_GREEN, TFT_BLACK);
      tft.setCursor (8, 52);
      tft.print("VERSION: 0.1"); // This uses the standard ADAFruit small font
      tft.setTextColor(0x39C4, TFT_BLACK);  // Leave a 7 segment ghost image, comment out next line!
      tft.drawString("88:88:88",0,0,7); // Overwrite the text to clear it
      
    }

    // Update digital time
    byte xpos = 6;
    byte ypos = 0;

    if (oss != ss) { // Only redraw every minute to minimise flicker
      // Uncomment ONE of the next 2 lines, using the ghost image demonstrates text overlay as time is drawn over it
      tft.setTextColor(0x39C4, TFT_BLACK);  // Leave a 7 segment ghost image, comment out next line!
      //tft.setTextColor(TFT_BLACK, TFT_BLACK); // Set font colour to black to wipe image
      // Font 7 is to show a pseudo 7 segment display.
      // Font 7 only contains characters [space] 0 1 2 3 4 5 6 7 8 9 0 : .
      tft.drawString("88:88:88",xpos,ypos,7); // Overwrite the text to clear it
      tft.setTextColor(0xFBE0); // Orange
      oss = ss;

      if (hh<10) 
        xpos+= tft.drawChar('0',xpos,ypos,7);
      xpos+= tft.drawNumber(hh,xpos,ypos,7);
      xcolon=xpos;
      xpos+= tft.drawChar(':',xpos,ypos,7);

      if (mm<10) 
        xpos+= tft.drawChar('0',xpos,ypos,7);
      xpos+= tft.drawNumber(mm,xpos,ypos,7);
      xpos+= tft.drawChar(':',xpos,ypos,7);

      if (ss<10) 
        xpos+= tft.drawChar('0',xpos,ypos,7);
      xpos+= tft.drawNumber(ss,xpos,ypos,7);
      

    


      tft.drawChar(':',xcolon,ypos,7);
      colour = random(0xFFFF);
      // Erase the old text with a rectangle, the disadvantage of this method is increased display flicker
      tft.fillRect (0, 60, 320, 160, TFT_BLACK);

      uint16_t v = analogRead(ADC_PIN);
      float battery_voltage = ((float)v / 4095.0) * 2.0 * 3.3 * (vref / 1000.0);
      String voltage = "BATT(V): " + String(battery_voltage) + "V";
      tft.drawString(voltage,0,64,4);

      uint16_t l = analogRead(ADC_PHOTO);
      float photo_voltage = ((float)l / 4095.0) * 2.0 * 3.3 * (vref / 1000.0);
      String photo = "PH(V): " + String(photo_voltage) + "V";
      tft.drawString(photo,0,100,4);

      if(photo_voltage > 4.0)
        off_cntr--;
      else
        off_cntr=5;
    
      if(off_cntr<=0) 
        RUN_FLAG=false;
      
      if(RUN_FLAG) {
        tft.setTextColor(0x7E0, TFT_BLACK);
        tft.drawRightString("RUN",240,100,4);
      } else {
        tft.setTextColor(0xF800, TFT_BLACK);
        tft.drawRightString("STOP",240,100,4);
      }
    }

    
  }
}



